# Copyright 2015 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.compute import security_group_default_rules_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestSecurityGroupDefaultRulesClient(base.BaseServiceTest):
    FAKE_RULE = {
        "from_port": 80,
        "id": 1,
        "ip_protocol": "TCP",
        "ip_range": {
            "cidr": "10.10.10.0/24"
        },
        "to_port": 80
    }

    def setUp(self):
        super(TestSecurityGroupDefaultRulesClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = (security_group_default_rules_client.
                       SecurityGroupDefaultRulesClient(fake_auth, 'compute',
                                                       'regionOne'))

    def _test_list_security_group_default_rules(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_security_group_default_rules,
            'tempest.lib.common.rest_client.RestClient.get',
            {"security_group_default_rules": [self.FAKE_RULE]},
            to_utf=bytes_body)

    def test_list_security_group_default_rules_with_str_body(self):
        self._test_list_security_group_default_rules()

    def test_list_security_group_default_rules_with_bytes_body(self):
        self._test_list_security_group_default_rules(bytes_body=True)

    def _test_show_security_group_default_rule(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_security_group_default_rule,
            'tempest.lib.common.rest_client.RestClient.get',
            {"security_group_default_rule": self.FAKE_RULE},
            to_utf=bytes_body,
            security_group_default_rule_id=1)

    def test_show_security_group_default_rule_with_str_body(self):
        self._test_show_security_group_default_rule()

    def test_show_security_group_default_rule_with_bytes_body(self):
        self._test_show_security_group_default_rule(bytes_body=True)

    def _test_create_security_default_group_rule(self, bytes_body=False):
        request_body = {
            "to_port": 80,
            "from_port": 80,
            "ip_protocol": "TCP",
            "cidr": "10.10.10.0/24"
        }
        self.check_service_client_function(
            self.client.create_security_default_group_rule,
            'tempest.lib.common.rest_client.RestClient.post',
            {"security_group_default_rule": self.FAKE_RULE},
            to_utf=bytes_body, **request_body)

    def test_create_security_default_group_rule_with_str_body(self):
        self._test_create_security_default_group_rule()

    def test_create_security_default_group_rule_with_bytes_body(self):
        self._test_create_security_default_group_rule(bytes_body=True)

    def test_delete_security_group_default_rule(self):
        self.check_service_client_function(
            self.client.delete_security_group_default_rule,
            'tempest.lib.common.rest_client.RestClient.delete',
            {}, status=204, security_group_default_rule_id=1)



# Copyright 2013 NTT Corporation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import tarfile
import tempfile

from tempest.api.object_storage import base
from tempest.common import object_storage
from tempest.common import utils
from tempest.lib import decorators


class BulkTest(base.BaseObjectTest):
    """Test bulk operation of archived file"""

    def setUp(self):
        super(BulkTest, self).setUp()
        self.containers = []

    def tearDown(self):
        # NOTE(andreaf) BulkTests needs to cleanup containers after each
        # test is executed.
        object_storage.delete_containers(self.containers,
                                         self.container_client,
                                         self.object_client)
        super(BulkTest, self).tearDown()

    def _create_archive(self):
        # Create an archived file for bulk upload testing.
        # Directory and files contained in the directory correspond to
        # container and subsidiary objects.
        tmp_dir = tempfile.mkdtemp()
        tmp_file = tempfile.mkstemp(dir=tmp_dir)

        # Extract a container name and an object name
        container_name = tmp_dir.split("/")[-1]
        object_name = tmp_file[1].split("/")[-1]

        # Create tar file
        tarpath = tempfile.NamedTemporaryFile(suffix=".tar")
        tar = tarfile.open(None, 'w', tarpath)
        tar.add(tmp_dir, arcname=container_name)
        tar.close()
        tarpath.flush()

        return tarpath.name, container_name, object_name

    def _upload_archive(self, filepath):
        # upload an archived file
        with open(filepath) as fh:
            mydata = fh.read()
            resp = self.bulk_client.upload_archive(
                upload_path='', data=mydata, archive_file_format='tar')
        return resp

    def _check_contents_deleted(self, container_name):
        param = {'format': 'txt'}
        resp, body = self.account_client.list_account_containers(param)
        self.assertHeaders(resp, 'Account', 'GET')
        self.assertNotIn(container_name, body)

    @decorators.idempotent_id('a407de51-1983-47cc-9f14-47c2b059413c')
    @utils.requires_ext(extension='bulk_upload', service='object')
    def test_extract_archive(self):
        """Test bulk operation of file upload with an archived file"""
        filepath, container_name, object_name = self._create_archive()
        resp = self._upload_archive(filepath)
        self.containers.append(container_name)

        self.assertHeaders(resp.response, 'Account', 'PUT')

        param = {'format': 'json'}
        resp, body = self.account_client.list_account_containers(param)

        self.assertHeaders(resp, 'Account', 'GET')

        self.assertIn(container_name, [b['name'] for b in body])

        param = {'format': 'json'}
        resp, contents_list = self.container_client.list_container_objects(
            container_name, param)

        self.assertHeaders(resp, 'Container', 'GET')

        self.assertIn(object_name, [c['name'] for c in contents_list])

    @decorators.idempotent_id('c075e682-0d2a-43b2-808d-4116200d736d')
    @utils.requires_ext(extension='bulk_delete', service='object')
    def test_bulk_delete(self):
        """Test bulk operation of deleting multiple files"""
        filepath, container_name, object_name = self._create_archive()
        self._upload_archive(filepath)

        data = '%s/%s\n%s' % (container_name, object_name, container_name)
        resp = self.bulk_client.delete_bulk_data(data=data)

        self.assertHeaders(resp.response, 'Account', 'DELETE')

        # Check if uploaded contents are completely deleted
        self._check_contents_deleted(container_name)

    @decorators.idempotent_id('dbea2bcb-efbb-4674-ac8a-a5a0e33d1d79')
    @utils.requires_ext(extension='bulk_delete', service='object')
    def test_bulk_delete_by_POST(self):
        """Test bulk operation of deleting multiple files by HTTP POST"""
        filepath, container_name, object_name = self._create_archive()
        self._upload_archive(filepath)

        data = '%s/%s\n%s' % (container_name, object_name, container_name)

        resp = self.bulk_client.delete_bulk_data_with_post(data=data)

        self.assertHeaders(resp.response, 'Account', 'POST')

        # Check if uploaded contents are completely deleted
        self._check_contents_deleted(container_name)



# Copyright 2017 AT&T Corporation.
# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.services.network import metering_labels_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestMeteringLabelsClient(base.BaseServiceTest):

    FAKE_METERING_LABELS = {
        "metering_labels": [
            {
                "project_id": "45345b0ee1ea477fac0f541b2cb79cd4",
                "tenant_id": "45345b0ee1ea477fac0f541b2cb79cd4",
                "description": "label1 description",
                "name": "label1",
                "id": "a6700594-5b7a-4105-8bfe-723b346ce866",
                "shared": False
            },
            {
                "project_id": "45345b0ee1ea477fac0f541b2cb79cd4",
                "tenant_id": "45345b0ee1ea477fac0f541b2cb79cd4",
                "description": "label2 description",
                "name": "label2",
                "id": "e131d186-b02d-4c0b-83d5-0c0725c4f812",
                "shared": False
            }
        ]
    }

    FAKE_METERING_LABEL_ID = "a6700594-5b7a-4105-8bfe-723b346ce866"

    def setUp(self):
        super(TestMeteringLabelsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.metering_labels_client = \
            metering_labels_client.MeteringLabelsClient(
                fake_auth, "network", "regionOne")

    def _test_list_metering_labels(self, bytes_body=False):
        self.check_service_client_function(
            self.metering_labels_client.list_metering_labels,
            "tempest.lib.common.rest_client.RestClient.get",
            self.FAKE_METERING_LABELS,
            bytes_body,
            200)

    def _test_create_metering_label(self, bytes_body=False):
        self.check_service_client_function(
            self.metering_labels_client.create_metering_label,
            "tempest.lib.common.rest_client.RestClient.post",
            {"metering_label": self.FAKE_METERING_LABELS[
                "metering_labels"][1]},
            bytes_body,
            201,
            name="label1",
            description="label1 description",
            shared=False)

    def _test_show_metering_label(self, bytes_body=False):
        self.check_service_client_function(
            self.metering_labels_client.show_metering_label,
            "tempest.lib.common.rest_client.RestClient.get",
            {"metering_label": self.FAKE_METERING_LABELS[
                "metering_labels"][0]},
            bytes_body,
            200,
            metering_label_id=self.FAKE_METERING_LABEL_ID)

    def test_delete_metering_label(self):
        self.check_service_client_function(
            self.metering_labels_client.delete_metering_label,
            "tempest.lib.common.rest_client.RestClient.delete",
            {},
            status=204,
            metering_label_id=self.FAKE_METERING_LABEL_ID)

    def test_list_metering_labels_with_str_body(self):
        self._test_list_metering_labels()

    def test_list_metering_labels_with_bytes_body(self):
        self._test_list_metering_labels(bytes_body=True)

    def test_create_metering_label_with_str_body(self):
        self._test_create_metering_label()

    def test_create_metering_label_with_bytes_body(self):
        self._test_create_metering_label(bytes_body=True)

    def test_show_metering_label_with_str_body(self):
        self._test_show_metering_label()

    def test_show_metering_label_with_bytes_body(self):
        self._test_show_metering_label(bytes_body=True)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest import config
from tempest.lib import decorators

CONF = config.CONF


# TODO(stephenfin): Remove these tests once the nova Ussuri branch goes EOL
class ServerMetadataTestJSON(base.BaseV2ComputeTest):
    """Test server metadata"""

    create_default_network = True

    @classmethod
    def setup_clients(cls):
        super(ServerMetadataTestJSON, cls).setup_clients()
        cls.client = cls.servers_client

    @classmethod
    def resource_setup(cls):
        super(ServerMetadataTestJSON, cls).resource_setup()
        cls.server = cls.create_test_server(metadata={}, wait_until='ACTIVE')

    def setUp(self):
        super(ServerMetadataTestJSON, self).setUp()
        meta = {'key1': 'value1', 'key2': 'value2'}
        self.client.set_server_metadata(self.server['id'], meta)

    @decorators.idempotent_id('479da087-92b3-4dcf-aeb3-fd293b2d14ce')
    def test_list_server_metadata(self):
        """Test listing server metadata

        All metadata key/value pairs for a server should be returned.
        """
        resp_metadata = (self.client.list_server_metadata(self.server['id'])
                         ['metadata'])

        # Verify the expected metadata items are in the list
        expected = {'key1': 'value1', 'key2': 'value2'}
        self.assertEqual(expected, resp_metadata)

    @decorators.idempotent_id('211021f6-21de-4657-a68f-908878cfe251')
    def test_set_server_metadata(self):
        """Test setting server metadata

        The server's metadata should be replaced with the provided values
        """
        # Create a new set of metadata for the server
        req_metadata = {'meta2': 'data2', 'meta3': 'data3'}
        self.client.set_server_metadata(self.server['id'], req_metadata)

        # Verify the expected values are correct, and that the
        # previous values have been removed
        resp_metadata = (self.client.list_server_metadata(self.server['id'])
                         ['metadata'])
        self.assertEqual(resp_metadata, req_metadata)

    @decorators.idempotent_id('344d981e-0c33-4997-8a5d-6c1d803e4134')
    def test_update_server_metadata(self):
        """Test updating server metadata

        The server's metadata values should be updated to the provided values.
        """
        meta = {'key1': 'alt1', 'key3': 'value3'}
        self.client.update_server_metadata(self.server['id'], meta)

        # Verify the values have been updated to the proper values
        resp_metadata = (self.client.list_server_metadata(self.server['id'])
                         ['metadata'])
        expected = {'key1': 'alt1', 'key2': 'value2', 'key3': 'value3'}
        self.assertEqual(expected, resp_metadata)

    @decorators.idempotent_id('0f58d402-e34a-481d-8af8-b392b17426d9')
    def test_update_metadata_empty_body(self):
        """Test updating server metadata to empty values

        The original server metadata should not be lost if empty metadata
        body is passed.
        """
        meta = {}
        self.client.update_server_metadata(self.server['id'], meta)
        resp_metadata = (self.client.list_server_metadata(self.server['id'])
                         ['metadata'])
        expected = {'key1': 'value1', 'key2': 'value2'}
        self.assertEqual(expected, resp_metadata)

    @decorators.idempotent_id('3043c57d-7e0e-49a6-9a96-ad569c265e6a')
    def test_get_server_metadata_item(self):
        """Test getting specific server metadata item"""
        meta = self.client.show_server_metadata_item(self.server['id'],
                                                     'key2')['meta']
        self.assertEqual('value2', meta['key2'])

    @decorators.idempotent_id('58c02d4f-5c67-40be-8744-d3fa5982eb1c')
    def test_set_server_metadata_item(self):
        """Test updating specific server metadata item

        The metadata item's value should be updated to the provided value.
        """

        # Update the metadata value.
        meta = {'nova': 'alt'}
        self.client.set_server_metadata_item(self.server['id'], 'nova', meta)

        # Verify the meta item's value has been updated
        resp_metadata = (self.client.list_server_metadata(self.server['id'])
                         ['metadata'])
        expected = {'key1': 'value1', 'key2': 'value2', 'nova': 'alt'}
        self.assertEqual(expected, resp_metadata)

    @decorators.idempotent_id('127642d6-4c7b-4486-b7cd-07265a378658')
    def test_delete_server_metadata_item(self):
        """Test deleting server metadata item

        The metadata value/key pair should be deleted from the server.
        """
        self.client.delete_server_metadata_item(self.server['id'], 'key1')

        # Verify the metadata item has been removed
        resp_metadata = (self.client.list_server_metadata(self.server['id'])
                         ['metadata'])
        expected = {'key2': 'value2'}
        self.assertEqual(expected, resp_metadata)



# Copyright 2016 NEC Corporation.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib.services.volume.v3 import snapshots_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestSnapshotsClient(base.BaseServiceTest):
    FAKE_CREATE_SNAPSHOT = {
        "snapshot": {
            "created_at": "2019-03-11T16:24:34.469003",
            "description": "Daily backup",
            "id": "b36476e5-d18b-47f9-ac69-4818cb43ee21",
            "metadata": {
                "key": "v3"
            },
            "name": "snap-001",
            "size": 10,
            "status": "creating",
            "updated_at": None,
            "volume_id": "d291b81c-6e40-4525-8231-90aa1588121e"
        }
    }

    FAKE_UPDATE_SNAPSHOT_RESPONSE = {
        "snapshot": {
            "created_at": "2019-03-12T04:53:53.426591",
            "description": "This is yet, another snapshot.",
            "id": "4a584cae-e4ce-429b-9154-d4c9eb8fda4c",
            "metadata": {
                "key": "v3"
            },
            "name": "snap-002",
            "size": 10,
            "status": "creating",
            "updated_at": None,
            "volume_id": "070c942d-9909-42e9-a467-7a781f150c58"
        }
    }

    FAKE_INFO_SNAPSHOT = {
        "snapshot": {
            "created_at": "2019-03-12T04:42:00.809352",
            "description": "Daily backup",
            "id": "4a584cae-e4ce-429b-9154-d4c9eb8fda4c",
            "metadata": {
                "key": "v3"
            },
            "name": "snap-001",
            "os-extended-snapshot-attributes:progress": "0%",
            "os-extended-snapshot-attributes:project_id":
                "89afd400-b646-4bbc-b12b-c0a4d63e5bd3",
            "size": 10,
            "status": "creating",
            "updated_at": None,
            "volume_id": "b72c48f1-64b7-4cd8-9745-b12e0be82d37"
        }
    }

    FAKE_LIST_SNAPSHOTS = {
        "snapshots": [
            {
                "created_at": "2019-03-11T16:24:36.464445",
                "description": "Daily backup",
                "id": "d0083dc5-8795-4c1a-bc9c-74f70006c205",
                "metadata": {
                    "key": "v3"
                },
                "name": "snap-001",
                "os-extended-snapshot-attributes:progress": "0%",
                "os-extended-snapshot-attributes:project_id":
                    "89afd400-b646-4bbc-b12b-c0a4d63e5bd3",
                "size": 10,
                "status": "creating",
                "updated_at": None,
                "volume_id": "7acd675e-4e06-4653-af9f-2ecd546342d6"
            }
        ]
    }

    FAKE_SNAPSHOT_METADATA_ITEM = {
        "metadata": {
            "key": "value"
        }
    }

    FAKE_SNAPSHOT_KEY = {
        "meta": {
            "key": "new_value"
        }
    }

    def setUp(self):
        super(TestSnapshotsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.client = snapshots_client.SnapshotsClient(fake_auth,
                                                       'volume',
                                                       'regionOne')

    def _test_create_snapshot(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_snapshot,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_CREATE_SNAPSHOT,
            bytes_body,
            status=202)

    def _test_show_snapshot(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_snapshot,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_INFO_SNAPSHOT,
            bytes_body,
            snapshot_id="4a584cae-e4ce-429b-9154-d4c9eb8fda4c")

    def _test_list_snapshots(self, bytes_body=False):
        self.check_service_client_function(
            self.client.list_snapshots,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_LIST_SNAPSHOTS,
            bytes_body,
            detail=True)

    def _test_create_snapshot_metadata(self, bytes_body=False):
        self.check_service_client_function(
            self.client.create_snapshot_metadata,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_SNAPSHOT_METADATA_ITEM,
            bytes_body,
            snapshot_id="4a584cae-e4ce-429b-9154-d4c9eb8fda4c",
            metadata={"key": "value"})

    def _test_update_snapshot(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_snapshot,
            'tempest.lib.common.rest_client.RestClient.put',
            self.FAKE_UPDATE_SNAPSHOT_RESPONSE,
            bytes_body,
            snapshot_id="4a584cae-e4ce-429b-9154-d4c9eb8fda4c")

    def _test_show_snapshot_metadata(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_snapshot_metadata,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_SNAPSHOT_METADATA_ITEM,
            bytes_body,
            snapshot_id="4a584cae-e4ce-429b-9154-d4c9eb8fda4c")

    def _test_update_snapshot_metadata(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_snapshot_metadata,
            'tempest.lib.common.rest_client.RestClient.put',
            self.FAKE_SNAPSHOT_METADATA_ITEM,
            bytes_body, snapshot_id="4a584cae-e4ce-429b-9154-d4c9eb8fda4c")

    def _test_update_snapshot_metadata_item(self, bytes_body=False):
        self.check_service_client_function(
            self.client.update_snapshot_metadata_item,
            'tempest.lib.common.rest_client.RestClient.put',
            self.FAKE_SNAPSHOT_KEY,
            bytes_body, volume_type_id="cbc36478b0bd8e67e89")

    def _test_show_snapshot_metadata_item(self, bytes_body=False):
        self.check_service_client_function(
            self.client.show_snapshot_metadata_item,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_SNAPSHOT_KEY,
            bytes_body,
            snapshot_id="4a584cae-e4ce-429b-9154-d4c9eb8fda4c",
            id="key1")

    def test_create_snapshot_with_str_body(self):
        self._test_create_snapshot()

    def test_create_snapshot_with_bytes_body(self):
        self._test_create_snapshot(bytes_body=True)

    def test_show_snapshot_with_str_body(self):
        self._test_show_snapshot()

    def test_show_snapshot_with_bytes_body(self):
        self._test_show_snapshot(bytes_body=True)

    def test_list_snapshots_with_str_body(self):
        self._test_list_snapshots()

    def test_list_snapshots_with_bytes_body(self):
        self._test_list_snapshots(bytes_body=True)

    def test_create_snapshot_metadata_with_str_body(self):
        self._test_create_snapshot_metadata()

    def test_create_snapshot_metadata_with_bytes_body(self):
        self._test_create_snapshot_metadata(bytes_body=True)

    def test_update_snapshot_with_str_body(self):
        self._test_update_snapshot()

    def test_update_snapshot_with_bytes_body(self):
        self._test_update_snapshot(bytes_body=True)

    def test_show_snapshot_metadata_with_str_body(self):
        self._test_show_snapshot_metadata()

    def test_show_snapshot_metadata_with_bytes_body(self):
        self._test_show_snapshot_metadata(bytes_body=True)

    def test_update_snapshot_metadata_with_str_body(self):
        self._test_update_snapshot_metadata()

    def test_update_snapshot_metadata_with_bytes_body(self):
        self._test_update_snapshot_metadata(bytes_body=True)

    def test_show_snapshot_metadata_item_with_str_body(self):
        self._test_show_snapshot_metadata_item()

    def test_show_snapshot_metadata_item_with_bytes_body(self):
        self._test_show_snapshot_metadata_item(bytes_body=True)

    def test_force_delete_snapshot(self):
        self.check_service_client_function(
            self.client.force_delete_snapshot,
            'tempest.lib.common.rest_client.RestClient.post',
            {},
            snapshot_id="521752a6-acf6-4b2d-bc7a-119f9148cd8c",
            status=202)

    def test_delete_snapshot(self):
        self.check_service_client_function(
            self.client.delete_snapshot,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            snapshot_id="521752a6-acf6-4b2d-bc7a-119f9148cd8c",
            status=202)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.services.network import segments_client
from tempest.tests.lib import fake_auth_provider
from tempest.tests.lib.services import base


class TestSegmentsClient(base.BaseServiceTest):

    FAKE_SEGMENT_ID = '83a59912-a473-11e9-a012-af494c35c9c2'
    FAKE_NETWORK_ID = '913ab0e4-a473-11e9-84a3-af1c16fc05de'

    FAKE_SEGMENT_REQUEST = {
        'segment': {
            'network_id': FAKE_NETWORK_ID,
            'segmentation_id': 2000,
            'network_type': 'vlan',
            'physical_network': 'segment-1'
        }
    }

    FAKE_SEGMENT_RESPONSE = {
        'segment': {
            'name': 'foo',
            'network_id': FAKE_NETWORK_ID,
            'segmentation_id': 2000,
            'network_type': 'vlan',
            'physical_network': 'segment-1',
            'revision_number': 1,
            'id': FAKE_SEGMENT_ID,
            'created_at': '2019-07-12T09:13:56Z',
            'updated_at': '2019-07-12T09:13:56Z',
            'description': 'bar'
        }
    }

    FAKE_SEGMENTS = {
        'segments': [
            FAKE_SEGMENT_RESPONSE['segment']
        ]
    }

    def setUp(self):
        super(TestSegmentsClient, self).setUp()
        fake_auth = fake_auth_provider.FakeAuthProvider()
        self.segments_client = segments_client.SegmentsClient(
            fake_auth, 'compute', 'regionOne')

    def _test_create_segment(self, bytes_body=False):
        self.check_service_client_function(
            self.segments_client.create_segment,
            'tempest.lib.common.rest_client.RestClient.post',
            self.FAKE_SEGMENT_RESPONSE,
            bytes_body,
            201,
            **self.FAKE_SEGMENT_REQUEST['segment']
        )

    def _test_list_segments(self, bytes_body=False):
        self.check_service_client_function(
            self.segments_client.list_segments,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_SEGMENTS,
            bytes_body,
            200
        )

    def _test_show_segment(self, bytes_body=False):
        self.check_service_client_function(
            self.segments_client.show_segment,
            'tempest.lib.common.rest_client.RestClient.get',
            self.FAKE_SEGMENT_RESPONSE,
            bytes_body,
            200,
            segment_id=self.FAKE_SEGMENT_ID
        )

    def _test_update_segment(self, bytes_body=False):
        update_kwargs = {
            'name': 'notfoo'
        }

        resp_body = {
            'segment': copy.deepcopy(self.FAKE_SEGMENT_RESPONSE['segment'])
        }
        resp_body['segment'].update(update_kwargs)

        self.check_service_client_function(
            self.segments_client.update_segment,
            'tempest.lib.common.rest_client.RestClient.put',
            resp_body,
            bytes_body,
            200,
            segment_id=self.FAKE_SEGMENT_ID,
            **update_kwargs
        )

    def test_create_segment_with_str_body(self):
        self._test_create_segment()

    def test_create_segment_with_bytes_body(self):
        self._test_create_segment(bytes_body=True)

    def test_update_segment_with_str_body(self):
        self._test_update_segment()

    def test_update_segment_with_bytes_body(self):
        self._test_update_segment(bytes_body=True)

    def test_show_segment_with_str_body(self):
        self._test_show_segment()

    def test_show_segment_with_bytes_body(self):
        self._test_show_segment(bytes_body=True)

    def test_delete_segment(self):
        self.check_service_client_function(
            self.segments_client.delete_segment,
            'tempest.lib.common.rest_client.RestClient.delete',
            {},
            status=204,
            segment_id=self.FAKE_SEGMENT_ID)

    def test_list_segment_with_str_body(self):
        self._test_list_segments()

    def test_list_segment_with_bytes_body(self):
        self._test_list_segments(bytes_body=True)



# Copyright 2013 Huawei Technologies Co.,LTD
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.volume import base
from tempest.common import waiters
from tempest import config
from tempest.lib import decorators

CONF = config.CONF


class SnapshotsActionsTest(base.BaseVolumeAdminTest):
    """Test volume snapshot actions"""

    @classmethod
    def skip_checks(cls):
        super(SnapshotsActionsTest, cls).skip_checks()
        if not CONF.volume_feature_enabled.snapshot:
            raise cls.skipException("Cinder snapshot feature disabled")

    @classmethod
    def resource_setup(cls):
        super(SnapshotsActionsTest, cls).resource_setup()

        # Create a test shared volume for tests
        cls.volume = cls.create_volume()

        # Create a test shared snapshot for tests
        cls.snapshot = cls.create_snapshot(volume_id=cls.volume['id'])

    def tearDown(self):
        # Set snapshot's status to available after test
        status = 'available'
        snapshot_id = self.snapshot['id']
        self.admin_snapshots_client.reset_snapshot_status(snapshot_id,
                                                          status)
        waiters.wait_for_volume_resource_status(self.snapshots_client,
                                                snapshot_id, status)
        super(SnapshotsActionsTest, self).tearDown()

    def _create_reset_and_force_delete_temp_snapshot(self, status=None):
        # Create snapshot, reset snapshot status,
        # and force delete temp snapshot
        temp_snapshot = self.create_snapshot(volume_id=self.volume['id'])
        if status:
            self.admin_snapshots_client.reset_snapshot_status(
                temp_snapshot['id'], status)
            waiters.wait_for_volume_resource_status(
                self.snapshots_client, temp_snapshot['id'], status)
        self.admin_snapshots_client.force_delete_snapshot(temp_snapshot['id'])
        self.snapshots_client.wait_for_resource_deletion(temp_snapshot['id'])

    def _get_progress_alias(self):
        return 'os-extended-snapshot-attributes:progress'

    @decorators.idempotent_id('3e13ca2f-48ea-49f3-ae1a-488e9180d535')
    def test_reset_snapshot_status(self):
        """Test resetting snapshot status to creating"""
        status = 'creating'
        self.admin_snapshots_client.reset_snapshot_status(
            self.snapshot['id'], status)
        waiters.wait_for_volume_resource_status(self.snapshots_client,
                                                self.snapshot['id'], status)

    @decorators.idempotent_id('41288afd-d463-485e-8f6e-4eea159413eb')
    def test_update_snapshot_status(self):
        """Test updating snapshot

        Update snapshot status to 'error' and progress to '80%'.
        """
        # Reset snapshot status to creating
        status = 'creating'
        self.admin_snapshots_client.reset_snapshot_status(
            self.snapshot['id'], status)
        waiters.wait_for_volume_resource_status(self.snapshots_client,
                                                self.snapshot['id'], status)

        # Update snapshot status to error
        progress = '80%'
        status = 'error'
        progress_alias = self._get_progress_alias()
        self.snapshots_client.update_snapshot_status(self.snapshot['id'],
                                                     status=status,
                                                     progress=progress)
        snapshot_get = self.admin_snapshots_client.show_snapshot(
            self.snapshot['id'])['snapshot']
        self.assertEqual(status, snapshot_get['status'])
        self.assertEqual(progress, snapshot_get[progress_alias])

    @decorators.idempotent_id('05f711b6-e629-4895-8103-7ca069f2073a')
    def test_snapshot_force_delete_when_snapshot_is_creating(self):
        """Test force delete when status of snapshot is creating"""
        self._create_reset_and_force_delete_temp_snapshot('creating')

    @decorators.idempotent_id('92ce8597-b992-43a1-8868-6316b22a969e')
    def test_snapshot_force_delete_when_snapshot_is_deleting(self):
        """Test force delete when status of snapshot is deleting"""
        self._create_reset_and_force_delete_temp_snapshot('deleting')

    @decorators.idempotent_id('645a4a67-a1eb-4e8e-a547-600abac1525d')
    def test_snapshot_force_delete_when_snapshot_is_error(self):
        """Test force delete when status of snapshot is error"""
        self._create_reset_and_force_delete_temp_snapshot('error')

    @decorators.idempotent_id('bf89080f-8129-465e-9327-b2f922666ba5')
    def test_snapshot_force_delete_when_snapshot_is_error_deleting(self):
        """Test force delete when status of snapshot is error_deleting"""
        self._create_reset_and_force_delete_temp_snapshot('error_deleting')



# Copyright 2014 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.api_schema.response.compute.v2_1 import parameter_types

# Compute microversion 2.23:
# New attributes in 'migrations' list.
#    'migration_type'
#    'links'

list_migrations = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'migrations': {
                'type': 'array',
                'items': {
                    'type': 'object',
                    'properties': {
                        'id': {'type': 'integer'},
                        'status': {'type': ['string', 'null']},
                        'instance_uuid': {'type': ['string', 'null']},
                        'source_node': {'type': ['string', 'null']},
                        'source_compute': {'type': ['string', 'null']},
                        'dest_node': {'type': ['string', 'null']},
                        'dest_compute': {'type': ['string', 'null']},
                        'dest_host': {'type': ['string', 'null']},
                        'old_instance_type_id': {'type': ['integer', 'null']},
                        'new_instance_type_id': {'type': ['integer', 'null']},
                        'created_at': {'type': 'string'},
                        'updated_at': {'type': ['string', 'null']},
                        # New attributes in version 2.23
                        'migration_type': {'type': ['string', 'null']},
                        'links': parameter_types.links
                    },
                    'additionalProperties': False,
                    'required': [
                        'id', 'status', 'instance_uuid', 'source_node',
                        'source_compute', 'dest_node', 'dest_compute',
                        'dest_host', 'old_instance_type_id',
                        'new_instance_type_id', 'created_at', 'updated_at',
                        'migration_type'
                    ]
                }
            }
        },
        'additionalProperties': False,
        'required': ['migrations']
    }
}



# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.identity import base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class ProjectsNegativeTestJSON(base.BaseIdentityV3AdminTest):
    """Negative tests of projects"""

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('8d68c012-89e0-4394-8d6b-ccd7196def97')
    def test_project_delete_by_unauthorized_user(self):
        """Non-admin user should not be able to delete a project"""
        project = self.setup_test_project()
        self.assertRaises(
            lib_exc.Forbidden, self.non_admin_projects_client.delete_project,
            project['id'])


class ProjectsNegativeStaticTestJSON(base.BaseIdentityV3AdminTest):
    """Negative tests of projects

    These tests can be executed in clouds using the pre-provisioned users
    """

    # NOTE: force_tenant_isolation is true in the base class by default but
    # overridden to false here to allow test execution for clouds using the
    # pre-provisioned credentials provider.
    force_tenant_isolation = False

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('24c49279-45dd-4155-887a-cb738c2385aa')
    def test_list_projects_by_unauthorized_user(self):
        """Non-admin user should not be able to list projects"""
        self.assertRaises(lib_exc.Forbidden,
                          self.non_admin_projects_client.list_projects)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('874c3e84-d174-4348-a16b-8c01f599561b')
    def test_project_create_duplicate(self):
        """Project names should be unique"""
        project_name = data_utils.rand_name(
            name='project-dup', prefix=CONF.resource_name_prefix)
        self.setup_test_project(name=project_name)

        self.assertRaises(lib_exc.Conflict,
                          self.projects_client.create_project, project_name)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('8fba9de2-3e1f-4e77-812a-60cb68f8df13')
    def test_create_project_by_unauthorized_user(self):
        """Non-admin user should not be authorized to create a project"""
        project_name = data_utils.rand_name(
            name='project', prefix=CONF.resource_name_prefix)
        self.assertRaises(
            lib_exc.Forbidden, self.non_admin_projects_client.create_project,
            project_name)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('7828db17-95e5-475b-9432-9a51b4aa79a9')
    def test_create_project_with_empty_name(self):
        """Project name should not be empty"""
        self.assertRaises(lib_exc.BadRequest,
                          self.projects_client.create_project, name='')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('502b6ceb-b0c8-4422-bf53-f08fdb21e2f0')
    def test_create_projects_name_length_over_64(self):
        """Project name length should not be greater than 64 characters"""
        project_name = 'a' * 65
        self.assertRaises(lib_exc.BadRequest,
                          self.projects_client.create_project, project_name)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('7965b581-60c1-43b7-8169-95d4ab7fc6fb')
    def test_delete_non_existent_project(self):
        """Attempt to delete a non existent project should fail"""
        self.assertRaises(lib_exc.NotFound,
                          self.projects_client.delete_project,
                          data_utils.rand_uuid_hex())



# Copyright (c) 2014 The Johns Hopkins University/Applied Physics Laboratory
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import testtools

from tempest.common import utils
from tempest.common import waiters
from tempest import config
from tempest.lib import decorators
from tempest.scenario import manager

CONF = config.CONF


class TestEncryptedCinderVolumes(manager.EncryptionScenarioTest):

    """The test suite for encrypted cinder volumes

    This test is for verifying the functionality of encrypted cinder volumes.

    For both LUKS (v1 & v2) and cryptsetup encryption types, this test performs
    the following:

    * Boots an instance from an image (CONF.compute.image_ref)
    * Creates an encryption type (as admin)
    * Creates a volume of that encryption type (as a regular user)
    * Attaches and detaches the encrypted volume to the instance
    """

    @classmethod
    def skip_checks(cls):
        super(TestEncryptedCinderVolumes, cls).skip_checks()
        if not CONF.compute_feature_enabled.attach_encrypted_volume:
            raise cls.skipException('Encrypted volume attach is not supported')

    def launch_instance(self):
        return self.create_server(wait_until='SSHABLE')

    def attach_detach_volume(self, server, volume):
        attached_volume = self.nova_volume_attach(server, volume)
        self.nova_volume_detach(server, attached_volume)

    @decorators.idempotent_id('79165fb4-5534-4b9d-8429-97ccffb8f86e')
    @decorators.attr(type='slow')
    @utils.services('compute', 'volume', 'image')
    def test_encrypted_cinder_volumes_luks(self):
        """LUKs v1 decrypts volume through libvirt."""
        volume = self.create_encrypted_volume('luks',
                                              volume_type='luks',
                                              wait_until=None)
        server = self.launch_instance()
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume['id'], 'available')
        # The volume retrieved on creation has a non-up-to-date status.
        # Retrieval after it becomes active ensures correct details.
        volume = self.volumes_client.show_volume(volume['id'])['volume']

        self.attach_detach_volume(server, volume)

    @decorators.idempotent_id('7abec0a3-61a0-42a5-9e36-ad3138fb38b4')
    @testtools.skipIf(CONF.volume.storage_protocol == 'ceph',
                      'Ceph only supports LUKSv2 if doing host attach.')
    @decorators.attr(type='slow')
    @utils.services('compute', 'volume', 'image')
    def test_encrypted_cinder_volumes_luksv2(self):
        """LUKs v2 decrypts volume through os-brick."""
        volume = self.create_encrypted_volume('luks2',
                                              volume_type='luksv2',
                                              wait_until=None)
        server = self.launch_instance()
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume['id'], 'available')
        # The volume retrieved on creation has a non-up-to-date status.
        # Retrieval after it becomes active ensures correct details.
        volume = self.volumes_client.show_volume(volume['id'])['volume']

        self.attach_detach_volume(server, volume)

    @decorators.idempotent_id('cbc752ed-b716-4717-910f-956cce965722')
    @decorators.attr(type='slow')
    @utils.services('compute', 'volume', 'image')
    def test_encrypted_cinder_volumes_cryptsetup(self):
        volume = self.create_encrypted_volume('plain',
                                              volume_type='cryptsetup',
                                              wait_until=None)
        server = self.launch_instance()
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume['id'], 'available')
        # The volume retrieved on creation has a non-up-to-date status.
        # Retrieval after it becomes active ensures correct details.
        volume = self.volumes_client.show_volume(volume['id'])['volume']

        self.attach_detach_volume(server, volume)



# Copyright 2022 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client


class ImageCacheClient(rest_client.RestClient):
    api_version = "v2"

    def list_cache(self):
        """Lists all images in cache or queue. (Since Image API v2.14)

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/image/v2/?expanded=query-cache-status-detail#cache-manage
        """
        url = 'cache'
        resp, body = self.get(url)
        self.expected_success(200, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def cache_queue(self, image_id):
        """Queues image for caching. (Since Image API v2.14)

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/image/v2/?expanded=queue-image-detail#queue-image
        """
        url = 'cache/%s' % image_id
        resp, body = self.put(url, body=None)
        self.expected_success(202, resp.status)
        return rest_client.ResponseBody(resp, body=body)

    def cache_delete(self, image_id):
        """Deletes a image from cache. (Since Image API v2.14)

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/image/v2/?expanded=delete-image-from-cache-detail#delete-image-from-cache
        """
        url = 'cache/%s' % image_id
        resp, _ = self.delete(url)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp)

    def cache_clear(self, target=None):
        """Clears the cache and its queue. (Since Image API v2.14)

        For a full list of available parameters, please refer to the official
        API reference:
        https://docs.openstack.org/api-ref/image/v2/?expanded=clear-images-from-cache-detail#delete-image-from-cache
        """
        url = 'cache'
        headers = {}
        if target:
            headers['x-image-cache-clear-target'] = target
        resp, _ = self.delete(url, headers=headers)
        self.expected_success(204, resp.status)
        return rest_client.ResponseBody(resp)



# Copyright 2016 EasyStack.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.image import base
from tempest.lib import decorators


class MetadataSchemaTest(base.BaseV2ImageTest):
    """Test to get image metadata schema"""

    @decorators.idempotent_id('e9e44891-3cb8-3b40-a532-e0a39fea3dab')
    def test_get_metadata_namespace_schema(self):
        """Test to get image namespace schema"""
        body = self.schemas_client.show_schema("metadefs/namespace")
        self.assertEqual("namespace", body['name'])

    @decorators.idempotent_id('ffe44891-678b-3ba0-a3e2-e0a3967b3aeb')
    def test_get_metadata_namespaces_schema(self):
        """Test to get image namespaces schema"""
        body = self.schemas_client.show_schema("metadefs/namespaces")
        self.assertEqual("namespaces", body['name'])

    @decorators.idempotent_id('fde34891-678b-3b40-ae32-e0a3e67b6beb')
    def test_get_metadata_resource_type_schema(self):
        """Test to get image resource_type schema"""
        body = self.schemas_client.show_schema("metadefs/resource_type")
        self.assertEqual("resource_type_association", body['name'])

    @decorators.idempotent_id('dfe4a891-b38b-3bf0-a3b2-e03ee67b3a3a')
    def test_get_metadata_resources_types_schema(self):
        """Test to get image resource_types schema"""
        body = self.schemas_client.show_schema("metadefs/resource_types")
        self.assertEqual("resource_type_associations", body['name'])

    @decorators.idempotent_id('dff4a891-b38b-3bf0-a3b2-e03ee67b3a3b')
    def test_get_metadata_object_schema(self):
        """Test to get image object schema"""
        body = self.schemas_client.show_schema("metadefs/object")
        self.assertEqual("object", body['name'])

    @decorators.idempotent_id('dee4a891-b38b-3bf0-a3b2-e03ee67b3a3c')
    def test_get_metadata_objects_schema(self):
        """Test to get image objects schema"""
        body = self.schemas_client.show_schema("metadefs/objects")
        self.assertEqual("objects", body['name'])

    @decorators.idempotent_id('dae4a891-b38b-3bf0-a3b2-e03ee67b3a3d')
    def test_get_metadata_property_schema(self):
        """Test to get image property schema"""
        body = self.schemas_client.show_schema("metadefs/property")
        self.assertEqual("property", body['name'])

    @decorators.idempotent_id('dce4a891-b38b-3bf0-a3b2-e03ee67b3a3e')
    def test_get_metadata_properties_schema(self):
        """Test to get image properties schema"""
        body = self.schemas_client.show_schema("metadefs/properties")
        self.assertEqual("properties", body['name'])

    @decorators.idempotent_id('dde4a891-b38b-3bf0-a3b2-e03ee67b3a3e')
    def test_get_metadata_tag_schema(self):
        """Test to get image tag schema"""
        body = self.schemas_client.show_schema("metadefs/tag")
        self.assertEqual("tag", body['name'])

    @decorators.idempotent_id('cde4a891-b38b-3bf0-a3b2-e03ee67b3a3a')
    def test_get_metadata_tags_schema(self):
        """Test to get image tags schema"""
        body = self.schemas_client.show_schema("metadefs/tags")
        self.assertEqual("tags", body['name'])



# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from oslo_serialization import jsonutils as json

from tempest.api.identity import base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

CONF = config.CONF


class CredentialsTestJSON(base.BaseIdentityV3AdminTest):
    """Test keystone credentials"""

    # NOTE: force_tenant_isolation is true in the base class by default but
    # overridden to false here to allow test execution for clouds using the
    # pre-provisioned credentials provider.
    force_tenant_isolation = False

    @classmethod
    def resource_setup(cls):
        super(CredentialsTestJSON, cls).resource_setup()
        cls.projects = list()
        cls.creds_list = [['project_id', 'user_id', 'id'],
                          ['access', 'secret']]
        for _ in range(2):
            project = cls.projects_client.create_project(
                data_utils.rand_name(
                    name='project', prefix=CONF.resource_name_prefix),
                description=data_utils.rand_name(
                    name='project-desc',
                    prefix=CONF.resource_name_prefix))['project']
            cls.addClassResourceCleanup(
                cls.projects_client.delete_project, project['id'])
            cls.projects.append(project['id'])
        cls.user_body = cls.users_client.show_user(
            cls.os_primary.credentials.user_id)['user']

    def _delete_credential(self, cred_id):
        self.creds_client.delete_credential(cred_id)

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('7cd59bf9-bda4-4c72-9467-d21cab278355')
    def test_credentials_create_get_update_delete(self):
        """Test creating, getting, updating, deleting of credentials"""
        prefix = CONF.resource_name_prefix
        blob = '{"access": "%s", "secret": "%s"}' % (
            data_utils.rand_name(name='Access', prefix=prefix),
            data_utils.rand_name(name='Secret', prefix=prefix))
        cred = self.creds_client.create_credential(
            user_id=self.user_body['id'], project_id=self.projects[0],
            blob=blob, type='ec2')['credential']
        self.addCleanup(self._delete_credential, cred['id'])
        for value1 in self.creds_list[0]:
            self.assertIn(value1, cred)
        for value2 in self.creds_list[1]:
            self.assertIn(value2, cred['blob'])

        new_keys = [data_utils.rand_name(name='NewAccess', prefix=prefix),
                    data_utils.rand_name(name='NewSecret', prefix=prefix)]
        blob = '{"access": "%s", "secret": "%s"}' % (new_keys[0], new_keys[1])
        update_body = self.creds_client.update_credential(
            cred['id'], blob=blob, project_id=self.projects[1],
            type='ec2')['credential']
        update_body['blob'] = json.loads(update_body['blob'])
        self.assertEqual(cred['id'], update_body['id'])
        self.assertEqual(self.projects[1], update_body['project_id'])
        self.assertEqual(self.user_body['id'], update_body['user_id'])
        self.assertEqual(update_body['blob']['access'], new_keys[0])
        self.assertEqual(update_body['blob']['secret'], new_keys[1])

        get_body = self.creds_client.show_credential(cred['id'])['credential']
        get_body['blob'] = json.loads(get_body['blob'])
        for value1 in self.creds_list[0]:
            self.assertEqual(update_body[value1],
                             get_body[value1])
        for value2 in self.creds_list[1]:
            self.assertEqual(update_body['blob'][value2],
                             get_body['blob'][value2])

    @decorators.idempotent_id('13202c00-0021-42a1-88d4-81b44d448aab')
    def test_credentials_list_delete(self):
        """Test listing credentials"""
        created_cred_ids = list()
        fetched_cred_ids = list()
        prefix = CONF.resource_name_prefix

        for _ in range(2):
            blob = '{"access": "%s", "secret": "%s"}' % (
                data_utils.rand_name(name='Access', prefix=prefix),
                data_utils.rand_name(name='Secret', prefix=prefix))
            cred = self.creds_client.create_credential(
                user_id=self.user_body['id'], project_id=self.projects[0],
                blob=blob, type='ec2')['credential']
            created_cred_ids.append(cred['id'])
            self.addCleanup(self._delete_credential, cred['id'])

        creds = self.creds_client.list_credentials()['credentials']

        for i in creds:
            fetched_cred_ids.append(i['id'])
        missing_creds = [c for c in created_cred_ids
                         if c not in fetched_cred_ids]
        self.assertEmpty(missing_creds,
                         "Failed to find cred %s in fetched list" %
                         ', '.join(m_cred for m_cred in missing_creds))



# Copyright 2013 Huawei Technologies Co.,LTD.
# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.network import base
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class NetworksNegativeTestJSON(base.BaseNetworkTest):
    """Negative tests of network"""

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('9293e937-824d-42d2-8d5b-e985ea67002a')
    def test_show_non_existent_network(self):
        """Test showing non existent network"""
        non_exist_id = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound, self.networks_client.show_network,
                          non_exist_id)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('d746b40c-5e09-4043-99f7-cba1be8b70df')
    def test_show_non_existent_subnet(self):
        """Test showing non existent subnet"""
        non_exist_id = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound, self.subnets_client.show_subnet,
                          non_exist_id)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('a954861d-cbfd-44e8-b0a9-7fab111f235d')
    def test_show_non_existent_port(self):
        """Test showing non existent port"""
        non_exist_id = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound, self.ports_client.show_port,
                          non_exist_id)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('98bfe4e3-574e-4012-8b17-b2647063de87')
    def test_update_non_existent_network(self):
        """Test updating non existent network"""
        non_exist_id = data_utils.rand_uuid()
        self.assertRaises(
            lib_exc.NotFound, self.networks_client.update_network,
            non_exist_id, name="new_name")

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('03795047-4a94-4120-a0a1-bd376e36fd4e')
    def test_delete_non_existent_network(self):
        """Test deleting non existent network"""
        non_exist_id = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound,
                          self.networks_client.delete_network,
                          non_exist_id)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('1cc47884-ac52-4415-a31c-e7ce5474a868')
    def test_update_non_existent_subnet(self):
        """Test updating non existent subnet"""
        non_exist_id = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound, self.subnets_client.update_subnet,
                          non_exist_id, name='new_name')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('a176c859-99fb-42ec-a208-8a85b552a239')
    def test_delete_non_existent_subnet(self):
        """Test deleting non existent subnet"""
        non_exist_id = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound,
                          self.subnets_client.delete_subnet, non_exist_id)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('13d3b106-47e6-4b9b-8d53-dae947f092fe')
    def test_create_port_on_non_existent_network(self):
        """Test creating port on non existent network"""
        non_exist_net_id = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound,
                          self.ports_client.create_port,
                          network_id=non_exist_net_id,
                          name=data_utils.rand_name(
                              self.__class__.__name__,
                              prefix=CONF.resource_name_prefix))

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('cf8eef21-4351-4f53-adcd-cc5cb1e76b92')
    def test_update_non_existent_port(self):
        """Test updating non existent port"""
        non_exist_port_id = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound, self.ports_client.update_port,
                          non_exist_port_id, name='new_name')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('49ec2bbd-ac2e-46fd-8054-798e679ff894')
    def test_delete_non_existent_port(self):
        """Test deleting non existent port"""
        non_exist_port_id = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound,
                          self.ports_client.delete_port, non_exist_port_id)



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import random
import re
import time
import zlib

from oslo_utils.secretutils import md5
from tempest.api.object_storage import base
from tempest.common import custom_matchers
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators

CONF = config.CONF


class ObjectTest(base.BaseObjectTest):
    """Test storage object"""

    @classmethod
    def resource_setup(cls):
        super(ObjectTest, cls).resource_setup()
        cls.container_name = cls.create_container()

    @classmethod
    def resource_cleanup(cls):
        cls.delete_containers()
        super(ObjectTest, cls).resource_cleanup()

    def _upload_segments(self):
        # create object
        object_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='LObject')
        data = data_utils.arbitrary_string()
        segments = 10
        data_segments = [data + str(i) for i in range(segments)]
        # uploading segments
        for i in range(segments):
            obj_name = "%s/%s" % (object_name, i)
            self.object_client.create_object(
                self.container_name, obj_name, data_segments[i])

        return object_name, data_segments

    def _copy_object_2d(self, src_object_name, metadata=None):
        dst_object_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='TestObject')
        resp, _ = self.object_client.copy_object_2d_way(self.container_name,
                                                        src_object_name,
                                                        dst_object_name,
                                                        metadata=metadata)
        return dst_object_name, resp

    def _check_copied_obj(self, dst_object_name, src_body,
                          in_meta=None, not_in_meta=None):
        resp, dest_body = self.object_client.get_object(self.container_name,
                                                        dst_object_name)

        self.assertEqual(src_body, dest_body)
        if in_meta:
            for meta_key in in_meta:
                self.assertIn('x-object-meta-' + meta_key, resp)
        if not_in_meta:
            for meta_key in not_in_meta:
                self.assertNotIn('x-object-meta-' + meta_key, resp)

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('5b4ce26f-3545-46c9-a2ba-5754358a4c62')
    def test_create_object(self):
        """Test creating object and checking the object's uploaded content"""
        # create object
        object_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='TestObject')
        data = data_utils.random_bytes()
        resp, _ = self.object_client.create_object(self.container_name,
                                                   object_name, data)
        # create another object
        object_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='TestObject')
        data = data_utils.random_bytes()
        resp, _ = self.object_client.create_object(self.container_name,
                                                   object_name, data)
        self.assertHeaders(resp, 'Object', 'PUT')

        # check uploaded content
        _, body = self.object_client.get_object(self.container_name,
                                                object_name)
        self.assertEqual(data, body)

    @decorators.idempotent_id('5daebb1d-f0d5-4dc9-b541-69672eff00b0')
    def test_create_object_with_content_disposition(self):
        """Test creating object with content-disposition"""
        object_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='TestObject')
        data = data_utils.random_bytes()
        metadata = {}
        metadata['content-disposition'] = 'inline'
        resp, _ = self.object_client.create_object(
            self.container_name,
            object_name,
            data,
            metadata=metadata)
        self.assertHeaders(resp, 'Object', 'PUT')

        resp, body = self.object_client.get_object(
            self.container_name,
            object_name,
            metadata=None)
        self.assertIn('content-disposition', resp)
        self.assertEqual(resp['content-disposition'], 'inline')
        self.assertEqual(body, data)

    @decorators.idempotent_id('605f8317-f945-4bee-ae91-013f1da8f0a0')
    def test_create_object_with_content_encoding(self):
        """Test creating object with content-encoding"""
        object_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='TestObject')

        # put compressed string
        data_before = b'x' * 2000
        data = zlib.compress(data_before)
        metadata = {}
        metadata['content-encoding'] = 'deflate'

        resp, _ = self.object_client.create_object(
            self.container_name,
            object_name,
            data,
            metadata=metadata)
        self.assertHeaders(resp, 'Object', 'PUT')

        # download compressed object
        metadata = {}
        metadata['accept-encoding'] = 'deflate'
        resp, body = self.object_client.get_object(
            self.container_name,
            object_name,
            metadata=metadata)
        self.assertEqual(body, data_before)

    @decorators.idempotent_id('73820093-0503-40b1-a478-edf0e69c7d1f')
    def test_create_object_with_etag(self):
        """Test creating object with Etag"""
        object_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='TestObject')
        data = data_utils.random_bytes()
        create_md5 = md5(data, usedforsecurity=False).hexdigest()
        metadata = {'Etag': create_md5}
        resp, _ = self.object_client.create_object(
            self.container_name,
            object_name,
            data,
            metadata=metadata)
        self.assertHeaders(resp, 'Object', 'PUT')

        # check uploaded content
        _, body = self.object_client.get_object(self.container_name,
                                                object_name)
        self.assertEqual(data, body)

    @decorators.idempotent_id('84dafe57-9666-4f6d-84c8-0814d37923b8')
    def test_create_object_with_expect_continue(self):
        """Test creating object with expect_continue"""
        object_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='TestObject')
        data = data_utils.random_bytes()

        status, _ = self.object_client.create_object_continue(
            self.container_name, object_name, data)

        self.assertEqual(status, 201)

        # check uploaded content
        _, body = self.object_client.get_object(self.container_name,
                                                object_name)
        self.assertEqual(data, body)

    @decorators.idempotent_id('4f84422a-e2f2-4403-b601-726a4220b54e')
    def test_create_object_with_transfer_encoding(self):
        """Test creating object with transfer_encoding"""
        object_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='TestObject')
        data = data_utils.random_bytes(1024)
        resp, _ = self.object_client.create_object(
            self.container_name,
            object_name,
            data=data_utils.chunkify(data, 512),
            chunked=True)

        self.assertHeaders(resp, 'Object', 'PUT')

        # check uploaded content
        _, body = self.object_client.get_object(self.container_name,
                                                object_name)
        self.assertEqual(data, body)

    @decorators.idempotent_id('0f3d62a6-47e3-4554-b0e5-1a5dc372d501')
    def test_create_object_with_x_fresh_metadata(self):
        """Test creating object with x-fresh-metadata

        The previous added metadata will be cleared.
        """
        object_name_base = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='TestObject')
        data = data_utils.random_bytes()
        metadata_1 = {'X-Object-Meta-test-meta': 'Meta'}
        self.object_client.create_object(self.container_name,
                                         object_name_base,
                                         data,
                                         metadata=metadata_1)
        object_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='TestObject')
        metadata_2 = {'X-Copy-From': '%s/%s' % (self.container_name,
                                                object_name_base),
                      'X-Fresh-Metadata': 'true'}
        resp, _ = self.object_client.create_object(
            self.container_name,
            object_name,
            '',
            metadata=metadata_2)
        self.assertHeaders(resp, 'Object', 'PUT')

        resp, body = self.object_client.get_object(self.container_name,
                                                   object_name)
        self.assertNotIn('x-object-meta-test-meta', resp)
        self.assertEqual(data, body)

    @decorators.idempotent_id('1c7ed3e4-2099-406b-b843-5301d4811baf')
    def test_create_object_with_x_object_meta(self):
        """Test creating object with x-object-meta"""
        object_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='TestObject')
        data = data_utils.random_bytes()
        metadata = {'X-Object-Meta-test-meta': 'Meta'}
        resp, _ = self.object_client.create_object(
            self.container_name,
            object_name,
            data,
            metadata=metadata)
        self.assertHeaders(resp, 'Object', 'PUT')

        resp, body = self.object_client.get_object(self.container_name,
                                                   object_name)
        self.assertIn('x-object-meta-test-meta', resp)
        self.assertEqual(resp['x-object-meta-test-meta'], 'Meta')
        self.assertEqual(data, body)

    @decorators.idempotent_id('e4183917-33db-4153-85cc-4dacbb938865')
    def test_create_object_with_x_object_metakey(self):
        """Test creating object with the blank value of metadata"""
        object_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='TestObject')
        data = data_utils.random_bytes()
        metadata = {'X-Object-Meta-test-meta': ''}
        resp, _ = self.object_client.create_object(
            self.container_name,
            object_name,
            data,
            metadata=metadata)
        self.assertHeaders(resp, 'Object', 'PUT')

        resp, body = self.object_client.get_object(self.container_name,
                                                   object_name)
        self.assertIn('x-object-meta-test-meta', resp)
        self.assertEqual(resp['x-object-meta-test-meta'], '')
        self.assertEqual(data, body)

    @decorators.idempotent_id('ce798afc-b278-45de-a5ce-2ea124b98b99')
    def test_create_object_with_x_remove_object_meta(self):
        """Test creating object with x-remove-object-meta

        The metadata will be removed from the object.
        """
        object_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='TestObject')
        data = data_utils.random_bytes()
        metadata_add = {'X-Object-Meta-test-meta': 'Meta'}
        self.object_client.create_object(self.container_name,
                                         object_name,
                                         data,
                                         metadata=metadata_add)
        metadata_remove = {'X-Remove-Object-Meta-test-meta': 'Meta'}
        resp, _ = self.object_client.create_object(
            self.container_name,
            object_name,
            data,
            metadata=metadata_remove)
        self.assertHeaders(resp, 'Object', 'PUT')

        resp, body = self.object_client.get_object(self.container_name,
                                                   object_name)
        self.assertNotIn('x-object-meta-test-meta', resp)
        self.assertEqual(data, body)

    @decorators.idempotent_id('ad21e342-7916-4f9e-ab62-a1f885f2aaf9')
    def test_create_object_with_x_remove_object_metakey(self):
        """Test creating object with the blank value of remove metadata

        Creating object with blank metadata 'X-Remove-Object-Meta-test-meta',
        metadata 'x-object-meta-test-meta' will be removed from the object.
        """
        object_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='TestObject')
        data = data_utils.random_bytes()
        metadata_add = {'X-Object-Meta-test-meta': 'Meta'}
        self.object_client.create_object(self.container_name,
                                         object_name,
                                         data,
                                         metadata=metadata_add)
        metadata_remove = {'X-Remove-Object-Meta-test-meta': ''}
        resp, _ = self.object_client.create_object(
            self.container_name,
            object_name,
            data,
            metadata=metadata_remove)
        self.assertHeaders(resp, 'Object', 'PUT')

        resp, body = self.object_client.get_object(self.container_name,
                                                   object_name)
        self.assertNotIn('x-object-meta-test-meta', resp)
        self.assertEqual(data, body)

    @decorators.idempotent_id('17738d45-03bd-4d45-9e0b-7b2f58f98687')
    def test_delete_object(self):
        """Test deleting object"""
        object_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='TestObject')
        data = data_utils.random_bytes()
        resp, _ = self.object_client.create_object(self.container_name,
                                                   object_name, data)
        # delete object
        resp, _ = self.object_client.delete_object(self.container_name,
                                                   object_name)
        self.assertHeaders(resp, 'Object', 'DELETE')

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('7a94c25d-66e6-434c-9c38-97d4e2c29945')
    def test_update_object_metadata(self):
        """Test updating object metadata"""
        object_name, _ = self.create_object(self.container_name)

        metadata = {'X-Object-Meta-test-meta': 'Meta'}
        resp, _ = self.object_client.create_or_update_object_metadata(
            self.container_name,
            object_name,
            headers=metadata)
        self.assertHeaders(resp, 'Object', 'POST')

        resp, _ = self.object_client.list_object_metadata(
            self.container_name,
            object_name)
        self.assertIn('x-object-meta-test-meta', resp)
        self.assertEqual(resp['x-object-meta-test-meta'], 'Meta')

    @decorators.idempotent_id('48650ed0-c189-4e1e-ad6b-1d4770c6e134')
    def test_update_object_metadata_with_remove_metadata(self):
        """Test updating object metadata with remove metadata"""
        object_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='TestObject')
        data = data_utils.random_bytes()
        create_metadata = {'X-Object-Meta-test-meta1': 'Meta1'}
        self.object_client.create_object(self.container_name,
                                         object_name,
                                         data,
                                         metadata=create_metadata)

        update_metadata = {'X-Remove-Object-Meta-test-meta1': 'Meta1'}
        resp, _ = self.object_client.create_or_update_object_metadata(
            self.container_name,
            object_name,
            headers=update_metadata)
        self.assertHeaders(resp, 'Object', 'POST')

        resp, _ = self.object_client.list_object_metadata(
            self.container_name,
            object_name)
        self.assertNotIn('x-object-meta-test-meta1', resp)

    @decorators.idempotent_id('f726174b-2ded-4708-bff7-729d12ce1f84')
    def test_update_object_metadata_with_create_and_remove_metadata(self):
        """Test updating object with creation and deletion of metadata

        Update object with creation and deletion of metadata with one
        request, both operations will succeed.
        """
        # creation and deletion of metadata with one request
        object_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='TestObject')
        data = data_utils.random_bytes()
        create_metadata = {'X-Object-Meta-test-meta1': 'Meta1'}
        self.object_client.create_object(self.container_name,
                                         object_name,
                                         data,
                                         metadata=create_metadata)

        update_metadata = {'X-Object-Meta-test-meta2': 'Meta2',
                           'X-Remove-Object-Meta-test-meta1': 'Meta1'}
        resp, _ = self.object_client.create_or_update_object_metadata(
            self.container_name,
            object_name,
            headers=update_metadata)
        self.assertHeaders(resp, 'Object', 'POST')

        resp, _ = self.object_client.list_object_metadata(
            self.container_name,
            object_name)
        self.assertNotIn('x-object-meta-test-meta1', resp)
        self.assertIn('x-object-meta-test-meta2', resp)
        self.assertEqual(resp['x-object-meta-test-meta2'], 'Meta2')

    @decorators.idempotent_id('08854588-6449-4bb7-8cca-f2e1040f5e6f')
    def test_update_object_metadata_with_x_object_manifest(self):
        """Test updating object metadata with x_object_manifest"""
        # uploading segments
        object_name, _ = self._upload_segments()
        # creating a manifest file
        data_empty = ''
        self.object_client.create_object(self.container_name,
                                         object_name,
                                         data_empty,
                                         metadata=None)
        object_prefix = '%s/%s' % (self.container_name, object_name)
        update_metadata = {'X-Object-Manifest': object_prefix}
        resp, _ = self.object_client.create_or_update_object_metadata(
            self.container_name,
            object_name,
            headers=update_metadata)
        self.assertHeaders(resp, 'Object', 'POST')

        resp, _ = self.object_client.list_object_metadata(
            self.container_name,
            object_name)
        self.assertIn('x-object-manifest', resp)
        self.assertNotEmpty(resp['x-object-manifest'])

    @decorators.idempotent_id('0dbbe89c-6811-4d84-a2df-eca2bdd40c0e')
    def test_update_object_metadata_with_x_object_metakey(self):
        """Test updating object metadata with a blank value of metadata"""
        object_name, _ = self.create_object(self.container_name)

        update_metadata = {'X-Object-Meta-test-meta': ''}
        resp, _ = self.object_client.create_or_update_object_metadata(
            self.container_name,
            object_name,
            headers=update_metadata)
        self.assertHeaders(resp, 'Object', 'POST')

        resp, _ = self.object_client.list_object_metadata(
            self.container_name,
            object_name)
        self.assertIn('x-object-meta-test-meta', resp)
        self.assertEqual(resp['x-object-meta-test-meta'], '')

    @decorators.idempotent_id('9a88dca4-b684-425b-806f-306cd0e57e42')
    def test_update_object_metadata_with_x_remove_object_metakey(self):
        """Test updating object metadata with blank remove metadata value"""
        object_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='TestObject')
        data = data_utils.arbitrary_string()
        create_metadata = {'X-Object-Meta-test-meta': 'Meta'}
        self.object_client.create_object(self.container_name,
                                         object_name,
                                         data,
                                         metadata=create_metadata)

        update_metadata = {'X-Remove-Object-Meta-test-meta': ''}
        resp, _ = self.object_client.create_or_update_object_metadata(
            self.container_name,
            object_name,
            headers=update_metadata)
        self.assertHeaders(resp, 'Object', 'POST')

        resp, _ = self.object_client.list_object_metadata(
            self.container_name,
            object_name)
        self.assertNotIn('x-object-meta-test-meta', resp)

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('9a447cf6-de06-48de-8226-a8c6ed31caf2')
    def test_list_object_metadata(self):
        """Test listing object metadata"""
        object_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='TestObject')
        data = data_utils.random_bytes()
        metadata = {'X-Object-Meta-test-meta': 'Meta'}
        self.object_client.create_object(self.container_name,
                                         object_name,
                                         data,
                                         metadata=metadata)

        resp, _ = self.object_client.list_object_metadata(
            self.container_name,
            object_name)
        self.assertHeaders(resp, 'Object', 'HEAD')
        self.assertIn('x-object-meta-test-meta', resp)
        self.assertEqual(resp['x-object-meta-test-meta'], 'Meta')

    @decorators.idempotent_id('170fb90e-f5c3-4b1f-ae1b-a18810821172')
    def test_list_no_object_metadata(self):
        """Test listing object metadata for object without metadata"""
        object_name, _ = self.create_object(self.container_name)

        resp, _ = self.object_client.list_object_metadata(
            self.container_name,
            object_name)
        self.assertHeaders(resp, 'Object', 'HEAD')
        self.assertNotIn('x-object-meta-', str(resp))

    @decorators.idempotent_id('23a3674c-d6de-46c3-86af-ff92bfc8a3da')
    def test_list_object_metadata_with_x_object_manifest(self):
        """Test getting object metadata with x_object_manifest"""
        # uploading segments
        object_name, _ = self._upload_segments()
        # creating a manifest file
        object_prefix = '%s/%s' % (self.container_name, object_name)
        metadata = {'X-Object-Manifest': object_prefix}
        data_empty = ''
        resp, _ = self.object_client.create_object(
            self.container_name,
            object_name,
            data_empty,
            metadata=metadata)

        resp, _ = self.object_client.list_object_metadata(
            self.container_name,
            object_name)

        # Check only the existence of common headers with custom matcher
        self.assertThat(resp, custom_matchers.ExistsAllResponseHeaders(
                        'Object', 'HEAD'))
        self.assertIn('x-object-manifest', resp)

        # Etag value of a large object is enclosed in double-quotations.
        # This is a special case, therefore the formats of response headers
        # are checked without a custom matcher.
        self.assertTrue(resp['etag'].startswith('\"'))
        self.assertTrue(resp['etag'].endswith('\"'))
        self.assertTrue(resp['etag'].strip('\"').isalnum())
        self.assertTrue(re.match(r"^\d+\.?\d*\Z", resp['x-timestamp']))
        self.assertNotEmpty(resp['content-type'])
        self.assertTrue(re.match("^tx[0-9a-f]{21}-[0-9a-f]{10}.*",
                                 resp['x-trans-id']))
        self.assertNotEmpty(resp['date'])
        self.assertEqual(resp['accept-ranges'], 'bytes')
        self.assertEqual(resp['x-object-manifest'],
                         '%s/%s' % (self.container_name, object_name))

    @decorators.attr(type='smoke')
    @decorators.idempotent_id('02610ba7-86b7-4272-9ed8-aa8d417cb3cd')
    def test_get_object(self):
        """Test retrieving object's data (in response body)"""

        # create object
        object_name, data = self.create_object(self.container_name)
        # get object
        resp, body = self.object_client.get_object(self.container_name,
                                                   object_name)
        self.assertHeaders(resp, 'Object', 'GET')

        self.assertEqual(body, data)

    @decorators.idempotent_id('005f9bf6-e06d-41ec-968e-96c78e0b1d82')
    def test_get_object_with_metadata(self):
        """Test getting object with metadata"""
        object_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='TestObject')
        data = data_utils.random_bytes()
        metadata = {'X-Object-Meta-test-meta': 'Meta'}
        self.object_client.create_object(self.container_name,
                                         object_name,
                                         data,
                                         metadata=metadata)
        resp, body = self.object_client.get_object(
            self.container_name,
            object_name,
            metadata=None)
        self.assertHeaders(resp, 'Object', 'GET')
        self.assertIn('x-object-meta-test-meta', resp)
        self.assertEqual(resp['x-object-meta-test-meta'], 'Meta')
        self.assertEqual(body, data)

    @decorators.idempotent_id('05a1890e-7db9-4a6c-90a8-ce998a2bddfa')
    def test_get_object_with_range(self):
        """Test getting object with range"""
        object_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='TestObject')
        data = data_utils.random_bytes(100)
        self.object_client.create_object(self.container_name,
                                         object_name,
                                         data,
                                         metadata=None)
        rand_num = random.randint(3, len(data) - 1)
        metadata = {'Range': 'bytes=%s-%s' % (rand_num - 3, rand_num - 1)}
        resp, body = self.object_client.get_object(
            self.container_name,
            object_name,
            metadata=metadata)
        self.assertHeaders(resp, 'Object', 'GET')
        self.assertEqual(body, data[rand_num - 3: rand_num])

    @decorators.idempotent_id('11b4515b-7ba7-4ca8-8838-357ded86fc10')
    def test_get_object_with_x_object_manifest(self):
        """Test getting object with x_object_manifest"""

        # uploading segments
        object_name, data_segments = self._upload_segments()
        # creating a manifest file
        object_prefix = '%s/%s' % (self.container_name, object_name)
        metadata = {'X-Object-Manifest': object_prefix}
        data_empty = ''
        resp, body = self.object_client.create_object(
            self.container_name,
            object_name,
            data_empty,
            metadata=metadata)

        resp, body = self.object_client.get_object(
            self.container_name,
            object_name,
            metadata=None)

        # Check only the existence of common headers with custom matcher
        self.assertThat(resp, custom_matchers.ExistsAllResponseHeaders(
                        'Object', 'GET'))
        self.assertIn('x-object-manifest', resp)

        # Etag value of a large object is enclosed in double-quotations.
        # This is a special case, therefore the formats of response headers
        # are checked without a custom matcher.
        self.assertTrue(resp['etag'].startswith('\"'))
        self.assertTrue(resp['etag'].endswith('\"'))
        self.assertTrue(resp['etag'].strip('\"').isalnum())
        self.assertTrue(re.match(r"^\d+\.?\d*\Z", resp['x-timestamp']))
        self.assertNotEmpty(resp['content-type'])
        self.assertTrue(re.match("^tx[0-9a-f]{21}-[0-9a-f]{10}.*",
                                 resp['x-trans-id']))
        self.assertNotEmpty(resp['date'])
        self.assertEqual(resp['accept-ranges'], 'bytes')
        self.assertEqual(resp['x-object-manifest'],
                         '%s/%s' % (self.container_name, object_name))

        self.assertEqual(''.join(data_segments), body.decode())

    @decorators.idempotent_id('c05b4013-e4de-47af-be84-e598062b16fc')
    def test_get_object_with_if_match(self):
        """Test getting object with if_match"""
        object_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='TestObject')
        data = data_utils.random_bytes(10)
        create_md5 = md5(data, usedforsecurity=False).hexdigest()
        create_metadata = {'Etag': create_md5}
        self.object_client.create_object(self.container_name,
                                         object_name,
                                         data,
                                         metadata=create_metadata)

        list_metadata = {'If-Match': create_md5}
        resp, body = self.object_client.get_object(
            self.container_name,
            object_name,
            metadata=list_metadata)
        self.assertHeaders(resp, 'Object', 'GET')
        self.assertEqual(body, data)

    @decorators.idempotent_id('be133639-e5d2-4313-9b1f-2d59fc054a16')
    def test_get_object_with_if_modified_since(self):
        """Test getting object with if_modified_since"""
        object_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='TestObject')
        data = data_utils.random_bytes()
        time_now = time.time()
        self.object_client.create_object(self.container_name,
                                         object_name,
                                         data,
                                         metadata=None)

        http_date = time.ctime(time_now - 86400)
        list_metadata = {'If-Modified-Since': http_date}
        resp, body = self.object_client.get_object(
            self.container_name,
            object_name,
            metadata=list_metadata)
        self.assertHeaders(resp, 'Object', 'GET')
        self.assertEqual(body, data)

    @decorators.idempotent_id('641500d5-1612-4042-a04d-01fc4528bc30')
    def test_get_object_with_if_none_match(self):
        """Test getting object with if_none_match"""
        object_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='TestObject')
        data = data_utils.random_bytes()
        create_md5 = md5(data, usedforsecurity=False).hexdigest()
        create_metadata = {'Etag': create_md5}
        self.object_client.create_object(self.container_name,
                                         object_name,
                                         data,
                                         metadata=create_metadata)

        list_data = data_utils.random_bytes()
        list_md5 = md5(list_data, usedforsecurity=False).hexdigest()
        list_metadata = {'If-None-Match': list_md5}
        resp, body = self.object_client.get_object(
            self.container_name,
            object_name,
            metadata=list_metadata)
        self.assertHeaders(resp, 'Object', 'GET')
        self.assertEqual(body, data)

    @decorators.idempotent_id('0aa1201c-10aa-467a-bee7-63cbdd463152')
    def test_get_object_with_if_unmodified_since(self):
        """Test getting object with if_unmodified_since"""
        object_name, data = self.create_object(self.container_name)

        time_now = time.time()
        http_date = time.ctime(time_now + 86400)
        list_metadata = {'If-Unmodified-Since': http_date}
        resp, body = self.object_client.get_object(
            self.container_name,
            object_name,
            metadata=list_metadata)
        self.assertHeaders(resp, 'Object', 'GET')
        self.assertEqual(body, data)

    @decorators.idempotent_id('94587078-475f-48f9-a40f-389c246e31cd')
    def test_get_object_with_x_newest(self):
        """Test getting object with x_newest"""
        object_name, data = self.create_object(self.container_name)

        list_metadata = {'X-Newest': 'true'}
        resp, body = self.object_client.get_object(
            self.container_name,
            object_name,
            metadata=list_metadata)
        self.assertHeaders(resp, 'Object', 'GET')
        self.assertEqual(body, data)

    @decorators.idempotent_id('1a9ab572-1b66-4981-8c21-416e2a5e6011')
    def test_copy_object_in_same_container(self):
        """Test copying object to another object in same container"""
        # create source object
        src_object_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='SrcObject')
        src_data = data_utils.random_bytes(size=len(src_object_name) * 2)
        resp, _ = self.object_client.create_object(self.container_name,
                                                   src_object_name,
                                                   src_data)
        # create destination object
        dst_object_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='DstObject')
        dst_data = data_utils.random_bytes(size=len(dst_object_name) * 3)
        resp, _ = self.object_client.create_object(self.container_name,
                                                   dst_object_name,
                                                   dst_data)
        # copy source object to destination
        headers = {}
        headers['X-Copy-From'] = "%s/%s" % (str(self.container_name),
                                            str(src_object_name))
        resp, body = self.object_client.create_object(self.container_name,
                                                      dst_object_name,
                                                      data=None,
                                                      headers=headers)
        self.assertHeaders(resp, 'Object', 'PUT')

        # check data
        resp, body = self.object_client.get_object(self.container_name,
                                                   dst_object_name)
        self.assertEqual(body, src_data)

    @decorators.idempotent_id('2248abba-415d-410b-9c30-22dff9cd6e67')
    def test_copy_object_to_itself(self):
        """Test changing the content type of an existing object"""

        # create object
        object_name, _ = self.create_object(self.container_name)
        # get the old content type
        resp_tmp, _ = self.object_client.list_object_metadata(
            self.container_name, object_name)
        # change the content type of the object
        metadata = {'content-type': 'text/plain; charset=UTF-8'}
        self.assertNotEqual(resp_tmp['content-type'], metadata['content-type'])
        headers = {}
        headers['X-Copy-From'] = "%s/%s" % (str(self.container_name),
                                            str(object_name))
        resp, _ = self.object_client.create_object(self.container_name,
                                                   object_name,
                                                   data=None,
                                                   metadata=metadata,
                                                   headers=headers)
        self.assertHeaders(resp, 'Object', 'PUT')

        # check the content type
        resp, _ = self.object_client.list_object_metadata(self.container_name,
                                                          object_name)
        self.assertEqual(resp['content-type'], metadata['content-type'])

    @decorators.idempotent_id('06f90388-2d0e-40aa-934c-e9a8833e958a')
    def test_copy_object_2d_way(self):
        """Test copying object's data to the new object using COPY"""
        # create source object
        src_object_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='SrcObject')
        src_data = data_utils.random_bytes(size=len(src_object_name) * 2)
        resp, _ = self.object_client.create_object(self.container_name,
                                                   src_object_name, src_data)
        # create destination object
        dst_object_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='DstObject')
        dst_data = data_utils.random_bytes(size=len(dst_object_name) * 3)
        resp, _ = self.object_client.create_object(self.container_name,
                                                   dst_object_name, dst_data)
        # copy source object to destination
        resp, _ = self.object_client.copy_object_2d_way(self.container_name,
                                                        src_object_name,
                                                        dst_object_name)
        self.assertHeaders(resp, 'Object', 'COPY')
        self.assertEqual(
            resp['x-copied-from'],
            self.container_name + "/" + src_object_name)

        # check data
        self._check_copied_obj(dst_object_name, src_data)

    @decorators.idempotent_id('aa467252-44f3-472a-b5ae-5b57c3c9c147')
    def test_copy_object_across_containers(self):
        """Test copying object to another container"""
        # create a container to use as a source container
        src_container_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name='TestSourceContainer')
        self.container_client.update_container(src_container_name)
        self.containers.append(src_container_name)
        # create a container to use as a destination container
        dst_container_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix,
            name='TestDestinationContainer')
        self.container_client.update_container(dst_container_name)
        self.containers.append(dst_container_name)
        # create object in source container
        object_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='Object')
        data = data_utils.random_bytes(size=len(object_name) * 2)
        resp, _ = self.object_client.create_object(src_container_name,
                                                   object_name, data)
        # set object metadata
        meta_key = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='test')
        meta_value = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='MetaValue')
        orig_metadata = {'X-Object-Meta-' + meta_key: meta_value}
        resp, _ = self.object_client.create_or_update_object_metadata(
            src_container_name,
            object_name,
            headers=orig_metadata)
        self.assertHeaders(resp, 'Object', 'POST')

        # copy object from source container to destination container
        headers = {}
        headers['X-Copy-From'] = "%s/%s" % (str(src_container_name),
                                            str(object_name))
        resp, body = self.object_client.create_object(dst_container_name,
                                                      object_name,
                                                      data=None,
                                                      headers=headers)
        self.assertHeaders(resp, 'Object', 'PUT')

        # check if object is present in destination container
        resp, body = self.object_client.get_object(dst_container_name,
                                                   object_name)
        self.assertEqual(body, data)
        actual_meta_key = 'x-object-meta-' + meta_key
        self.assertIn(actual_meta_key, resp)
        self.assertEqual(resp[actual_meta_key], meta_value)

    @decorators.idempotent_id('5a9e2cc6-85b6-46fc-916d-0cbb7a88e5fd')
    def test_copy_object_with_x_fresh_metadata(self):
        """Test copying objectwith x_fresh_metadata"""
        # create source object
        metadata = {'x-object-meta-src': 'src_value'}
        src_object_name, data = self.create_object(self.container_name,
                                                   metadata=metadata)

        # copy source object with x_fresh_metadata header
        metadata = {'X-Fresh-Metadata': 'true'}
        dst_object_name, resp = self._copy_object_2d(src_object_name,
                                                     metadata)

        self.assertHeaders(resp, 'Object', 'COPY')

        self.assertNotIn('x-object-meta-src', resp)
        self.assertEqual(resp['x-copied-from'],
                         self.container_name + "/" + src_object_name)

        # check that destination object does NOT have any object-meta
        self._check_copied_obj(dst_object_name, data, not_in_meta=["src"])

    @decorators.idempotent_id('a28a8b99-e701-4d7e-9d84-3b66f121460b')
    def test_copy_object_with_x_object_metakey(self):
        """Test copying object with x_object_metakey"""
        # create source object
        metadata = {'x-object-meta-src': 'src_value'}
        src_obj_name, data = self.create_object(self.container_name,
                                                metadata=metadata)

        # copy source object to destination with x-object-meta-key
        metadata = {'x-object-meta-test': ''}
        dst_obj_name, resp = self._copy_object_2d(src_obj_name, metadata)

        self.assertHeaders(resp, 'Object', 'COPY')

        expected = {'x-object-meta-test': '',
                    'x-object-meta-src': 'src_value',
                    'x-copied-from': self.container_name + "/" + src_obj_name}
        for key, value in expected.items():
            self.assertIn(key, resp)
            self.assertEqual(value, resp[key])

        # check destination object
        self._check_copied_obj(dst_obj_name, data, in_meta=["test", "src"])

    @decorators.idempotent_id('edabedca-24c3-4322-9b70-d6d9f942a074')
    def test_copy_object_with_x_object_meta(self):
        """Test copying object with x_object_meta"""
        # create source object
        metadata = {'x-object-meta-src': 'src_value'}
        src_obj_name, data = self.create_object(self.container_name,
                                                metadata=metadata)

        # copy source object to destination with object metadata
        metadata = {'x-object-meta-test': 'value'}
        dst_obj_name, resp = self._copy_object_2d(src_obj_name, metadata)

        self.assertHeaders(resp, 'Object', 'COPY')

        expected = {'x-object-meta-test': 'value',
                    'x-object-meta-src': 'src_value',
                    'x-copied-from': self.container_name + "/" + src_obj_name}
        for key, value in expected.items():
            self.assertIn(key, resp)
            self.assertEqual(value, resp[key])

        # check destination object
        self._check_copied_obj(dst_obj_name, data, in_meta=["test", "src"])

    @decorators.idempotent_id('e3e6a64a-9f50-4955-b987-6ce6767c97fb')
    def test_object_upload_in_segments(self):
        """Test uploading object in segments"""
        # create object
        object_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='LObject')
        data = data_utils.arbitrary_string()
        segments = 10
        data_segments = [data + str(i) for i in range(segments)]
        # uploading segments
        for i in range(segments):
            obj_name = "%s/%s" % (object_name, i)
            resp, _ = self.object_client.create_object(
                self.container_name, obj_name, data_segments[i])
        # creating a manifest file
        metadata = {'X-Object-Manifest': '%s/%s/'
                    % (self.container_name, object_name)}
        resp, _ = self.object_client.create_object(self.container_name,
                                                   object_name, data='')
        self.assertHeaders(resp, 'Object', 'PUT')

        resp, _ = self.object_client.create_or_update_object_metadata(
            self.container_name, object_name, headers=metadata)
        self.assertHeaders(resp, 'Object', 'POST')

        resp, _ = self.object_client.list_object_metadata(
            self.container_name, object_name)

        # Etag value of a large object is enclosed in double-quotations.
        # After etag quotes are checked they are removed and the response is
        # checked if all common headers are present and well formatted
        self.assertTrue(resp['etag'].startswith('\"'))
        self.assertTrue(resp['etag'].endswith('\"'))
        resp['etag'] = resp['etag'].strip('"')
        self.assertHeaders(resp, 'Object', 'HEAD')

        self.assertIn('x-object-manifest', resp)
        self.assertEqual(resp['x-object-manifest'],
                         '%s/%s/' % (self.container_name, object_name))

        # downloading the object
        resp, body = self.object_client.get_object(
            self.container_name, object_name)
        self.assertEqual(''.join(data_segments), body.decode())

    @decorators.idempotent_id('50d01f12-526f-4360-9ac2-75dd508d7b68')
    def test_get_object_if_different(self):
        """Test getting object content only when the local file is different

        http://en.wikipedia.org/wiki/HTTP_ETag
        Make a conditional request for an object using the If-None-Match
        header, it should get downloaded only if the local file is different,
        otherwise the response code should be 304 Not Modified
        """
        object_name, data = self.create_object(self.container_name)
        # local copy is identical, no download
        object_md5 = md5(data, usedforsecurity=False).hexdigest()
        headers = {'If-None-Match': object_md5}
        url = "%s/%s" % (self.container_name, object_name)
        resp, _ = self.object_client.get(url, headers=headers)
        self.assertEqual(resp['status'], '304')

        # When the file is not downloaded from Swift server, response does
        # not contain 'X-Timestamp' header. This is the special case, therefore
        # the existence of response headers is checked without custom matcher.
        self.assertIn('date', resp)
        # Check only the format of common headers with custom matcher
        self.assertThat(resp, custom_matchers.AreAllWellFormatted())

        # local copy is different, download
        local_data = "something different"
        other_md5 = md5(local_data.encode(), usedforsecurity=False).hexdigest()
        headers = {'If-None-Match': other_md5}
        resp, _ = self.object_client.get(url, headers=headers)
        self.assertHeaders(resp, 'Object', 'GET')


class PublicObjectTest(base.BaseObjectTest):
    """Test public storage object"""

    credentials = [['operator', CONF.object_storage.operator_role],
                   ['operator_alt', CONF.object_storage.operator_role]]

    @classmethod
    def setup_credentials(cls):
        super(PublicObjectTest, cls).setup_credentials()
        cls.os_alt = cls.os_roles_operator_alt

    @classmethod
    def setup_clients(cls):
        super(PublicObjectTest, cls).setup_clients()
        cls.object_client_alt = cls.os_alt.object_client

    def setUp(self):
        super(PublicObjectTest, self).setUp()
        self.container_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='TestContainer')
        self.container_client.update_container(self.container_name)
        self.containers.append(self.container_name)

    def tearDown(self):
        self.delete_containers()
        super(PublicObjectTest, self).tearDown()

    @decorators.idempotent_id('07c9cf95-c0d4-4b49-b9c8-0ef2c9b27193')
    def test_access_public_container_object_without_using_creds(self):
        """Test accessing public container object without using credentials

        Make container public-readable and access an object in it object
        anonymously, without using credentials.
        """
        # update container metadata to make it publicly readable
        cont_headers = {'X-Container-Read': '.r:*,.rlistings'}
        resp_meta, body = (
            self.container_client.create_update_or_delete_container_metadata(
                self.container_name,
                create_update_metadata=cont_headers,
                create_update_metadata_prefix=''))
        self.assertHeaders(resp_meta, 'Container', 'POST')

        # create object
        object_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='Object')
        data = data_utils.random_bytes(size=len(object_name))
        resp, _ = self.object_client.create_object(self.container_name,
                                                   object_name, data)
        self.assertHeaders(resp, 'Object', 'PUT')

        # list container metadata
        resp_meta, _ = self.container_client.list_container_metadata(
            self.container_name)
        self.assertHeaders(resp_meta, 'Container', 'HEAD')

        self.assertIn('x-container-read', resp_meta)
        self.assertEqual(resp_meta['x-container-read'], '.r:*,.rlistings')

        # trying to get object with empty headers as it is public readable
        self.object_client.auth_provider.set_alt_auth_data(
            request_part='headers',
            auth_data=None
        )
        resp, body = self.object_client.get_object(
            self.container_name, object_name)
        self.assertHeaders(resp, 'Object', 'GET')

        self.assertEqual(body, data)

    @decorators.idempotent_id('54e2a2fe-42dc-491b-8270-8e4217dd4cdc')
    def test_access_public_object_with_another_user_creds(self):
        """Test accessing public object with another user's credentials

        Make container public-readable and access an object in it using
        another user's credentials.
        """
        cont_headers = {'X-Container-Read': '.r:*,.rlistings'}
        resp_meta, body = (
            self.container_client.create_update_or_delete_container_metadata(
                self.container_name, create_update_metadata=cont_headers,
                create_update_metadata_prefix=''))
        self.assertHeaders(resp_meta, 'Container', 'POST')

        # create object
        object_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='Object')
        data = data_utils.random_bytes(size=len(object_name))
        resp, _ = self.object_client.create_object(self.container_name,
                                                   object_name, data)
        self.assertHeaders(resp, 'Object', 'PUT')

        # list container metadata
        resp, _ = self.container_client.list_container_metadata(
            self.container_name)
        self.assertHeaders(resp, 'Container', 'HEAD')

        self.assertIn('x-container-read', resp)
        self.assertEqual(resp['x-container-read'], '.r:*,.rlistings')

        # get auth token of alternative user
        alt_auth_data = self.object_client_alt.auth_provider.auth_data
        self.object_client.auth_provider.set_alt_auth_data(
            request_part='headers',
            auth_data=alt_auth_data
        )
        # access object using alternate user creds
        resp, body = self.object_client.get_object(
            self.container_name, object_name)
        self.assertHeaders(resp, 'Object', 'GET')

        self.assertEqual(body, data)



# Copyright 2018 ZTE Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from tempest.lib.api_schema.response.compute.v2_36 import quotas as quotasv236

# Compute microversion 2.57:
# remove attributes in quota_set:
#    'injected_file_content_bytes',
#    'injected_file_path_bytes',
#    'injected_files'

remove_item_list = ['injected_file_content_bytes', 'injected_file_path_bytes',
                    'injected_files']

update_quota_set = copy.deepcopy(quotasv236.update_quota_set)
for item in remove_item_list:
    update_quota_set['response_body']['properties']['quota_set'][
        'properties'].pop(item)
    update_quota_set['response_body']['properties']['quota_set'][
        'required'].remove(item)

get_quota_set = copy.deepcopy(quotasv236.get_quota_set)
for item in remove_item_list:
    get_quota_set['response_body']['properties']['quota_set'][
        'properties'].pop(item)
    get_quota_set['response_body']['properties']['quota_set'][
        'required'].remove(item)

get_quota_set_details = copy.deepcopy(quotasv236.get_quota_set_details)
for item in remove_item_list:
    get_quota_set_details['response_body']['properties']['quota_set'][
        'properties'].pop(item)
    get_quota_set_details['response_body']['properties']['quota_set'][
        'required'].remove(item)

# NOTE(zhufl): Below are the unchanged schema in this microversion. We need
# to keep this schema in this file to have the generic way to select the
# right schema based on self.schema_versions_info mapping in service client.
# ****** Schemas unchanged since microversion 2.1 ***
delete_quota = copy.deepcopy(quotasv236.delete_quota)



# Copyright 2014 NEC Corporation.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

update_quota_set = {
    'status_code': [200],
    'response_body': {
        'type': 'object',
        'properties': {
            'quota_set': {
                'type': 'object',
                'properties': {
                    'instances': {'type': 'integer'},
                    'cores': {'type': 'integer'},
                    'ram': {'type': 'integer'},
                    'floating_ips': {'type': 'integer'},
                    'fixed_ips': {'type': 'integer'},
                    'metadata_items': {'type': 'integer'},
                    'key_pairs': {'type': 'integer'},
                    'security_groups': {'type': 'integer'},
                    'security_group_rules': {'type': 'integer'},
                    'server_group_members': {'type': 'integer'},
                    'server_groups': {'type': 'integer'},
                    'injected_files': {'type': 'integer'},
                    'injected_file_content_bytes': {'type': 'integer'},
                    'injected_file_path_bytes': {'type': 'integer'}
                },
                'additionalProperties': False,
                # NOTE: server_group_members and server_groups are represented
                # when enabling quota_server_group extension. So they should
                # not be required.
                'required': ['instances', 'cores', 'ram',
                             'floating_ips', 'fixed_ips',
                             'metadata_items', 'key_pairs',
                             'security_groups', 'security_group_rules',
                             'injected_files', 'injected_file_content_bytes',
                             'injected_file_path_bytes']
            }
        },
        'additionalProperties': False,
        'required': ['quota_set']
    }
}

get_quota_set = copy.deepcopy(update_quota_set)
get_quota_set['response_body']['properties']['quota_set']['properties'][
    'id'] = {'type': 'string'}
get_quota_set['response_body']['properties']['quota_set']['required'].extend([
    'id'])

get_quota_set_details = copy.deepcopy(get_quota_set)
get_quota_set_details['response_body']['properties']['quota_set'][
    'properties'] = {
    'id': {'type': 'string'},
    'instances': {
        'type': 'object',
        'properties': {
            'reserved': {'type': 'integer'},
            'limit': {'type': 'integer'},
            'in_use': {'type': 'integer'}
        }
    },
    'cores': {
        'type': 'object',
        'properties': {
            'reserved': {'type': 'integer'},
            'limit': {'type': 'integer'},
            'in_use': {'type': 'integer'}
        }
    },
    'ram': {
        'type': 'object',
        'properties': {
            'reserved': {'type': 'integer'},
            'limit': {'type': 'integer'},
            'in_use': {'type': 'integer'}
        }
    },
    'floating_ips': {
        'type': 'object',
        'properties': {
            'reserved': {'type': 'integer'},
            'limit': {'type': 'integer'},
            'in_use': {'type': 'integer'}
        }
    },
    'fixed_ips': {
        'type': 'object',
        'properties': {
            'reserved': {'type': 'integer'},
            'limit': {'type': 'integer'},
            'in_use': {'type': 'integer'}
        }
    },
    'metadata_items': {
        'type': 'object',
        'properties': {
            'reserved': {'type': 'integer'},
            'limit': {'type': 'integer'},
            'in_use': {'type': 'integer'}
        }
    },
    'key_pairs': {
        'type': 'object',
        'properties': {
            'reserved': {'type': 'integer'},
            'limit': {'type': 'integer'},
            'in_use': {'type': 'integer'}
        }
    },
    'security_groups': {
        'type': 'object',
        'properties': {
            'reserved': {'type': 'integer'},
            'limit': {'type': 'integer'},
            'in_use': {'type': 'integer'}
        }
    },
    'security_group_rules': {
        'type': 'object',
        'properties': {
            'reserved': {'type': 'integer'},
            'limit': {'type': 'integer'},
            'in_use': {'type': 'integer'}
        }
    },
    'server_group_members': {
        'type': 'object',
        'properties': {
            'reserved': {'type': 'integer'},
            'limit': {'type': 'integer'},
            'in_use': {'type': 'integer'}
        }
    },
    'server_groups': {
        'type': 'object',
        'properties': {
            'reserved': {'type': 'integer'},
            'limit': {'type': 'integer'},
            'in_use': {'type': 'integer'}
        }
    },
    'injected_files': {
        'type': 'object',
        'properties': {
            'reserved': {'type': 'integer'},
            'limit': {'type': 'integer'},
            'in_use': {'type': 'integer'}
        }
    },
    'injected_file_content_bytes': {
        'type': 'object',
        'properties': {
            'reserved': {'type': 'integer'},
            'limit': {'type': 'integer'},
            'in_use': {'type': 'integer'}
        }
    },
    'injected_file_path_bytes': {
        'type': 'object',
        'properties': {
            'reserved': {'type': 'integer'},
            'limit': {'type': 'integer'},
            'in_use': {'type': 'integer'}
        }
    }
}

delete_quota = {
    'status_code': [202]
}



# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_serialization import jsonutils as json

from tempest.lib.api_schema.response.compute.v2_1 import versions as schema
from tempest.lib.common import rest_client
from tempest.lib.services.compute import base_compute_client


class VersionsClient(base_compute_client.BaseComputeClient):

    def list_versions(self):
        version_url = self._get_base_version_url()

        resp, body = self.raw_request(version_url, 'GET')

        self._error_checker(resp, body)
        body = json.loads(body)
        self.validate_response(schema.list_versions, resp, body)
        return rest_client.ResponseBody(resp, body)

    def get_version_by_url(self, version_url):
        """Get the version document by url.

        This gets the version document for a url, useful in testing
        the contents of things like /v2/ or /v2.1/ in Nova. That
        controller needs authenticated access, so we have to get
        ourselves a token before making the request.

        """
        # we need a token for this request
        resp, body = self.raw_request(version_url, 'GET',
                                      {'X-Auth-Token': self.token})
        self._error_checker(resp, body)
        body = json.loads(body)
        self.validate_response(schema.get_one_version, resp, body)
        return rest_client.ResponseBody(resp, body)



# Copyright 2012 OpenStack Foundation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.api.compute import base
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc

CONF = config.CONF


class ImagesNegativeTestBase(base.BaseV2ComputeTest):

    @classmethod
    def skip_checks(cls):
        super(ImagesNegativeTestBase, cls).skip_checks()
        if not CONF.service_available.glance:
            skip_msg = ("%s skipped as glance is not available" % cls.__name__)
            raise cls.skipException(skip_msg)
        if not CONF.compute_feature_enabled.snapshot:
            skip_msg = ("%s skipped as instance snapshotting is not supported"
                        % cls.__name__)
            raise cls.skipException(skip_msg)

    @classmethod
    def setup_clients(cls):
        super(ImagesNegativeTestBase, cls).setup_clients()
        cls.client = cls.compute_images_client


class ImagesNegativeTestJSON(ImagesNegativeTestBase):
    """Negative tests of server image"""
    create_default_network = True

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('6cd5a89d-5b47-46a7-93bc-3916f0d84973')
    def test_create_image_from_deleted_server(self):
        """Check server image should not be created if the server is removed"""
        server = self.create_test_server(wait_until='ACTIVE')

        # Delete server before trying to create image
        self.servers_client.delete_server(server['id'])
        waiters.wait_for_server_termination(self.servers_client, server['id'])
        # Create a new image after server is deleted
        meta = {'image_type': 'test'}
        self.assertRaises(lib_exc.NotFound,
                          self.create_image_from_server,
                          server['id'], metadata=meta)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('82c5b0c4-9dbd-463c-872b-20c4755aae7f')
    def test_create_image_from_invalid_server(self):
        """Check server image should not be created with invalid server id"""
        # Create a new image with invalid server id
        meta = {'image_type': 'test'}
        self.assertRaises(
            lib_exc.NotFound,
            self.create_image_from_server,
            data_utils.rand_name(prefix=CONF.resource_name_prefix,
                                 name='invalid'),
            metadata=meta)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('ec176029-73dc-4037-8d72-2e4ff60cf538')
    def test_create_image_specify_uuid_35_characters_or_less(self):
        """Check server image should not be created for invalid server id

        Return an error if server id passed is 35 characters or less
        """
        snapshot_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='test-snap')
        test_uuid = ('a' * 35)
        self.assertRaises(lib_exc.NotFound, self.client.create_image,
                          test_uuid, name=snapshot_name)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('36741560-510e-4cc2-8641-55fe4dfb2437')
    def test_create_image_specify_uuid_37_characters_or_more(self):
        """Check server image should not be created for invalid server id

        Return an error if sever id passed is 37 characters or more
        """
        snapshot_name = data_utils.rand_name(
            prefix=CONF.resource_name_prefix, name='test-snap')
        test_uuid = ('a' * 37)
        self.assertRaises(lib_exc.NotFound, self.client.create_image,
                          test_uuid, name=snapshot_name)


class ImagesDeleteNegativeTestJSON(ImagesNegativeTestBase):
    """Negative tests of server image

    Negative tests of server image with compute microversion less than 2.36.
    """
    max_microversion = '2.35'

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('381acb65-785a-4942-94ce-d8f8c84f1f0f')
    def test_delete_image_with_invalid_image_id(self):
        """Check an image should not be deleted with invalid image id"""
        self.assertRaises(
            lib_exc.NotFound, self.client.delete_image,
            data_utils.rand_name(
                prefix=CONF.resource_name_prefix, name='invalid'))

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('137aef61-39f7-44a1-8ddf-0adf82511701')
    def test_delete_non_existent_image(self):
        """Check trying to delete a non-existent image should fail"""
        non_existent_image_id = data_utils.rand_uuid()
        self.assertRaises(lib_exc.NotFound, self.client.delete_image,
                          non_existent_image_id)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('e6e41425-af5c-4fe6-a4b5-7b7b963ffda5')
    def test_delete_image_blank_id(self):
        """Check trying to delete an image with blank id should fail"""
        self.assertRaises(lib_exc.NotFound, self.client.delete_image, '')

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('924540c3-f1f1-444c-8f58-718958b6724e')
    def test_delete_image_non_hex_string_id(self):
        """Check trying to delete an image with non hex id should fail"""
        invalid_image_id = data_utils.rand_uuid()[:-1] + "j"
        self.assertRaises(lib_exc.NotFound, self.client.delete_image,
                          invalid_image_id)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('68e2c175-bd26-4407-ac0f-4ea9ce2139ea')
    def test_delete_image_negative_image_id(self):
        """Check trying to delete an image with negative id should fail"""
        self.assertRaises(lib_exc.NotFound, self.client.delete_image, -1)

    @decorators.attr(type=['negative'])
    @decorators.idempotent_id('b340030d-82cd-4066-a314-c72fb7c59277')
    def test_delete_image_with_id_over_character_limit(self):
        """Check trying to delete image with id over limit should fail"""
        invalid_image_id = data_utils.rand_uuid() + "1"
        self.assertRaises(lib_exc.NotFound, self.client.delete_image,
                          invalid_image_id)
